// ACME - a crossassembler for producing 6502/65c02/65816 code.
// Copyright (C) 1998-2014 Marco Baye
// Have a look at "acme.c" for further info
//
// ALU stuff (the expression parser)
#ifndef alu_H
#define alu_H


#include "config.h"


// constants

// meaning of bits in "flags" of struct result:
#define MVALUE_IS_FP	(1u << 8)	// floating point value
#define MVALUE_INDIRECT	(1u << 7)	// needless parentheses indicate use of indirect addressing modes
#define MVALUE_EXISTS	(1u << 6)	// 0: expression was empty. 1: there was *something* to parse.
#define MVALUE_UNSURE	(1u << 5)	// value once was related to undefined
// expression. Needed for producing the same addresses in all passes; because in
// the first pass there will almost for sure be labels that are undefined, you
// can't simply get the addressing mode from looking at the parameter's value.
#define MVALUE_DEFINED	(1u << 4)	// 0: undefined expression (value will be zero). 1: known result
#define MVALUE_ISBYTE	(1u << 3)	// value is guaranteed to fit in one byte
#define MVALUE_FORCE24	(1u << 2)	// value usage forces 24-bit usage
#define MVALUE_FORCE16	(1u << 1)	// value usage forces 16-bit usage
#define MVALUE_FORCE08	(1u << 0)	// value usage forces 8-bit usage
#define MVALUE_FORCEBITS	(MVALUE_FORCE08|MVALUE_FORCE16|MVALUE_FORCE24)
#define MVALUE_GIVEN	(MVALUE_DEFINED | MVALUE_EXISTS)	// bit mask for fixed values (defined and existing)


// create dynamic buffer, operator/function trees and operator/operand stacks
extern void ALU_init(void);
// activate error output for "value undefined"
extern void ALU_throw_errors(void);
// returns int value (0 if result was undefined)
extern intval_t ALU_any_int(void);
// returns int value (if result was undefined, serious error is thrown)
extern intval_t ALU_defined_int(void);
// stores int value if given. Returns whether stored. Throws error if undefined.
extern int ALU_optional_defined_int(intval_t *target);
// stores int value and flags (floats are transformed to int)
extern void ALU_int_result(struct result *intresult);
// stores int value and flags, allowing for one '(' too many (x-indirect addr).
// returns number of additional '(' (1 or 0).
extern int ALU_liberal_int(struct result *intresult);
// stores value and flags (result may be either int or float)
extern void ALU_any_result(struct result *result);


#endif
